/*
 * This file is part of SEVino.
 *
 * SEVino is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEVino is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SEVino.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

/*
 *  TODO:
 *  enhance shotdetection (be more fading robust, e.g., earth_short.avi)
 *  better scrolling/disabling of settings if e.g. constant or variable is selected
 *  
 *  INFO:
 *  Validation removed because always java heap overflow!!!!
 */

package app;

import io.LoadSEM;
import io.SaveSEM;
import io.UploadFile;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Timer;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.SwingWorker;
import javax.swing.WindowConstants;
import javax.swing.JLabel;
import javax.swing.border.EtchedBorder;

import lib.AVFileFilter;
import lib.AuthorInfo;
import lib.BarElement;
import lib.ConfigElement;
import lib.EffectTrack;
import lib.IntField;
import lib.Painter;
import lib.Ruler;
import lib.RulerBorder;
import lib.SEMFileFilter;
import lib.GuiUpdater;
import lib.ShotsThread;
import lib.SliderToolTips;
import lib.VideoPainter;
import lib.EffectPanel;

import javax.swing.GroupLayout;
import javax.swing.LayoutStyle;


/**
 * The application's main frame.
 * 
 * @author Markus Waltl
 * @version 1.0
 */
public class SEVinoView extends JFrame implements WindowListener, ActionListener,
																		MouseListener, AdjustmentListener {
	private final static String applicationName_ = "SEVino 1.1";
	private SEVinoView gui = this;
	private static final long serialVersionUID = -7111871466518602855L;
	private JButton forwardButton;
    private JPanel mainPanel;
    private JButton openVideo;
    public JButton playButton;
    private JPanel playback;
    private JButton backwardButton;
    private JButton rewindButton;
    private JButton saveSEM;
    private JButton stopButton;
    private JButton exitApp;
    private JButton uploadButton;
    private JButton statisticButton;
    private JPanel timeLineNormal;
    private JPanel timeLineShots;
    public JSlider timeSlider;
    private JPanel tracksPanel;
    private JPanel infoPanel;
    public VideoPainter videoPanel;
    private JButton pauseButton;
    private String curVideoFile = "";
    public JLabel durationLabel;
    public JLabel curTimeLabel;
    public JLabel codecLabel;
    public JLabel bitrateLabel;
    public JLabel resolutionLabel;
    private JLabel lblDuration;
    private JLabel lblCurTime;
    private JLabel lblCodec;
    private JLabel lblBitrate;
    private JLabel lblResolution;
    private JPanel appControls;
    public JPanel shotPanel;
    private JScrollPane shotPanelPane;
    private JProgressBar progressBar;
    public GuiUpdater guiUpdater_ = null;
    public Vector<Double> timeList_ = null;    
    public Vector<Painter> shotPanels = new Vector<Painter>();
    private JScrollPane effectLayerPane;
    private JPanel leftEffectPanel;
    public JScrollPane rightEffectLayer;
    public JPanel rightEffectPanel;
    private JPanel effectPanel;
    public JScrollBar trackHorizontalScroll;
    private JButton authorButton;
    private AuthorInfo authorInfo;
    public Vector<EffectTrack> effectTracks_ = null;
    private JButton loadSEM;
    private JButton clearBars;
    private Timer timer_ = null;
    public JScrollPane rulerPane;
    public Ruler ruler;
    private ShotsThread shotsThread = null;
    
    public JSlider volumeSlider;
    private JLabel volumeLabel;
    
    
    private String preSetServerAddress = "localhost";
    private int preSetServerPort = 8080;
    private String preSetServerPath = "/cms/fileUpload.do";
    private String preSetFilename = "testsem.xml";
      
    private final String SERVERINI = "configs/server.ini";
    
    private String saveSEMPath = null;
    private String loadSEMPath = null;
    private String loadVideoPath = null;
        
    /**
     * Constructor
     * @param configElements Vector with configuration elements
     */
    public SEVinoView(Vector<ConfigElement> configElements) {
        initComponents(configElements);
        setLocation(0, 0);
		setSize(860, 768);
		addWindowListener(this);
    }

    /**
     * Method for initializing the components
     * @param configElements Vector with configuration elements
     */
    private void initComponents(Vector<ConfigElement> configElements) {
        mainPanel = new JPanel();
        videoPanel = new VideoPainter();
        timeLineNormal = new JPanel();
        timeSlider = new JSlider(0, 0, 0);
        timeLineShots = new JPanel();
        tracksPanel = new JPanel();
        infoPanel = new JPanel();
        playback = new JPanel();
        playButton = new JButton();
        backwardButton = new JButton();
        forwardButton = new JButton();
        stopButton = new JButton();
        pauseButton = new JButton();
        openVideo = new JButton();
        saveSEM = new JButton();
        exitApp = new JButton();
        uploadButton = new JButton();
        appControls = new JPanel();
        rewindButton = new JButton();
        statisticButton = new JButton();
        shotPanel = new JPanel();
        shotPanelPane = new JScrollPane(shotPanel);
        progressBar = new JProgressBar(0, 100);
        trackHorizontalScroll = new JScrollBar(JScrollBar.HORIZONTAL);
        authorButton = new JButton();
        loadSEM = new JButton();
        clearBars = new JButton();
        
        volumeSlider = new JSlider(0, 200, 100);
        volumeLabel = new JLabel("Vol:");
                
        durationLabel = new JLabel();
        curTimeLabel = new JLabel();
        codecLabel = new JLabel();
        bitrateLabel = new JLabel();
        resolutionLabel = new JLabel();
        lblDuration = new JLabel("Duration: ");
        lblCurTime = new JLabel("Current time: ");
        lblCodec = new JLabel("Codec: ");
        lblBitrate = new JLabel("Bitrate: ");
        lblResolution = new JLabel("Resolution: ");
        
        effectPanel = new JPanel();
        effectLayerPane = new JScrollPane(effectPanel);
        leftEffectPanel = new JPanel();
        rightEffectPanel = new JPanel();
        rightEffectLayer = new JScrollPane(rightEffectPanel);
        
        ruler = new Ruler(this);
        ruler.setBounds(0, 0, 730, 22);
        ruler.setPreferredSize(new Dimension(730, 22));
        ruler.setName("ruler");
        rulerPane = new JScrollPane(ruler);
        
        mainPanel.setName("mainPanel");
        
        appControls.setBorder(javax.swing.BorderFactory.createTitledBorder("App controls"));
        appControls.setName("appControls");
        
        openVideo.setText("Open A/V");
        openVideo.setName("openVideo");
        openVideo.setToolTipText("Open video file");
        openVideo.setActionCommand("openVideo");
        openVideo.addActionListener(this);
        
        saveSEM.setText("Generate");
        saveSEM.setName("saveSEM");
        saveSEM.setToolTipText("Generate SEM file");
        saveSEM.setActionCommand("saveSEM");
        saveSEM.addActionListener(this);
        
        exitApp.setText("Exit");
        exitApp.setActionCommand("exitApp");
        exitApp.setName("exitApp");
        exitApp.setToolTipText("Exit application");
        exitApp.addActionListener(this);
        
        uploadButton.setText("Upload");
        uploadButton.setActionCommand("uploadSEM");
        uploadButton.setName("uploadSEM");
        uploadButton.setToolTipText("Generate and upload SEM");
        uploadButton.addActionListener(this);        
        
        authorButton.setText("AuthorInfo");
        authorButton.setActionCommand("authorButton");
        authorButton.setName("authorButton");
        authorButton.setToolTipText("Information about author");
        authorButton.addActionListener(this);
        
        loadSEM.setText("Load SEM");
        loadSEM.setActionCommand("loadSEM");
        loadSEM.setName("loadSEM");
        loadSEM.setToolTipText("Load SEM description");
        loadSEM.addActionListener(this);    
        
        clearBars.setText("Clear Bars");
        clearBars.setActionCommand("clearBars");
        clearBars.setName("clearBars");
        clearBars.setToolTipText("Clear edit bars");
        clearBars.addActionListener(this);      
        
        statisticButton.setText("Statistics");
        statisticButton.setActionCommand("statisticsSEM");
        statisticButton.setName("statisticsSEM");
        statisticButton.setToolTipText("Show how many effects are annotated");
        statisticButton.addActionListener(this);      
        
        
        progressBar.setValue(100);
        progressBar.setPreferredSize(new Dimension(60,10));
        
        GroupLayout appControlsLayout = new GroupLayout(appControls);
        appControls.setLayout(appControlsLayout);
        appControlsLayout.setHorizontalGroup(
        		appControlsLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
                .addGroup(appControlsLayout.createParallelGroup()
                .addComponent(openVideo)
                .addComponent(authorButton)
                .addComponent(saveSEM)
                .addComponent(uploadButton)
                .addComponent(loadSEM)
                .addComponent(clearBars)
                .addComponent(statisticButton)
                .addComponent(exitApp)
                .addComponent(progressBar))
            );
        
        appControlsLayout.linkSize(SwingConstants.HORIZONTAL, new Component[] {openVideo, authorButton, saveSEM, uploadButton, loadSEM, clearBars, statisticButton, exitApp, progressBar});
        
        final int d = 3;
        appControlsLayout.setVerticalGroup(
        		appControlsLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
                .addGroup(appControlsLayout.createSequentialGroup()
                    .addComponent(openVideo)
                    .addGap(d,d,d)
                    .addComponent(authorButton)
                    .addGap(d,d,d)
                    .addComponent(saveSEM)
                    .addGap(d,d,d)
                    .addComponent(uploadButton)
                    .addGap(d,d,d)
                    .addComponent(loadSEM)
                	.addGap(d,d,d)
                	.addComponent(clearBars)
                	.addGap(d,d,d)
                	.addComponent(statisticButton)
                	.addGap(d,d,d)
                    .addComponent(exitApp)
                    .addGap(20, 20, 20)
                    .addComponent(progressBar))
            );
        
        videoPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Video"));
        videoPanel.setName("videoPanel"); 

        GroupLayout videoPanelLayout = new GroupLayout(videoPanel);
        videoPanel.setLayout(videoPanelLayout);
        videoPanelLayout.setHorizontalGroup(
            videoPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGap(0, 474, Short.MAX_VALUE)
        );
        videoPanelLayout.setVerticalGroup(
            videoPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGap(0, 255, Short.MAX_VALUE)
        );

        timeLineNormal.setBorder(javax.swing.BorderFactory.createTitledBorder("Timeline"));
        timeLineNormal.setName("timeLineNormal");

        timeSlider.setName("timeSlider");
        timeSlider.setEnabled(false);
		SliderToolTips.enableSliderToolTips(timeSlider, this);

        GroupLayout timeLineNormalLayout = new GroupLayout(timeLineNormal);
        timeLineNormal.setLayout(timeLineNormalLayout);
        timeLineNormalLayout.setHorizontalGroup(
            timeLineNormalLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addComponent(timeSlider, GroupLayout.DEFAULT_SIZE, 900, Short.MAX_VALUE)
        );
        timeLineNormalLayout.setVerticalGroup(
            timeLineNormalLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGroup(timeLineNormalLayout.createSequentialGroup()
                .addComponent(timeSlider, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
                .addContainerGap(GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        timeLineShots.setBorder(javax.swing.BorderFactory.createTitledBorder("Shot-Timeline"));
        timeLineShots.setName("timeLineShots");
        
        
        shotPanelPane.setName("shotPanelPane");
        shotPanelPane.setBounds(0, 0, 900, 70);
        shotPanelPane.setPreferredSize(new Dimension(900, 70));
        
        shotPanel.setName("shotPanel");
        shotPanel.setBounds(0, 0, 800, 50);
        shotPanel.setPreferredSize(new Dimension(800, 50));
        
        GroupLayout timeLineShotsLayout = new GroupLayout(timeLineShots);
        timeLineShots.setLayout(timeLineShotsLayout);
        timeLineShotsLayout.setHorizontalGroup(
            timeLineShotsLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addComponent(shotPanelPane, GroupLayout.DEFAULT_SIZE, 900, Short.MAX_VALUE)
            
        );
        timeLineShotsLayout.setVerticalGroup(
            timeLineShotsLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGroup(timeLineShotsLayout.createSequentialGroup()
            	.addComponent(shotPanelPane, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
        );

        tracksPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Effect tracks"));
        tracksPanel.setName("tracksPanel");
        

        effectLayerPane.setName("effectLayerPane");
        effectLayerPane.setBounds(0, 0, 900, 232);
        effectLayerPane.setPreferredSize(new Dimension(900, 232));
        effectLayerPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        effectLayerPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        rulerPane.setName("rulerPane");
        rulerPane.setBounds(0, 0, 710, 22);
        rulerPane.setPreferredSize(new Dimension(710, 22));
        rulerPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        //rulerPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
        rulerPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
        rulerPane.setBorder(null);
                
        // needed because ScrollPane can only get one panel
        effectPanel.setName("effectPanel");
        effectPanel.setBounds(0, 0, 835, 220);
        effectPanel.setPreferredSize(new Dimension(835, 220));
                
        leftEffectPanel.setName("leftEffectPanel");
        leftEffectPanel.setBounds(0, 0, 100, 232);
        leftEffectPanel.setPreferredSize(new Dimension(100, 232));
        //leftEffectPanel.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        leftEffectPanel.setLayout(new FlowLayout(FlowLayout.CENTER, 0, 0));

        // needed because ScrollPane can only get one panel
        rightEffectLayer.setName("rightEffectLayer");
        rightEffectLayer.setBounds(0, 0, 725, 232);
        rightEffectLayer.setPreferredSize(new Dimension(725, 232));
        rightEffectLayer.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
        rightEffectLayer.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        rightEffectLayer.setBorder(null);
        //rightEffectLayer.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        
        rightEffectPanel.setName("rightEffectPanel");
        rightEffectPanel.setBounds(0, 0, 720, 220);
        rightEffectPanel.setPreferredSize(new Dimension(720, 220));
        rightEffectPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
        
        trackHorizontalScroll.setName("trackHorizontalScroll");
        trackHorizontalScroll.setBounds(0, 0, 725, 16);
        trackHorizontalScroll.setPreferredSize(new Dimension(725, 16));
        trackHorizontalScroll.setMaximum(0);
        trackHorizontalScroll.addAdjustmentListener(this);
        trackHorizontalScroll.setEnabled(false);
        
        GroupLayout effectPanelLayout = new GroupLayout(effectPanel);
        effectPanel.setLayout(effectPanelLayout);
        effectPanelLayout.setHorizontalGroup(
        	effectPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
        	.addGroup(effectPanelLayout.createSequentialGroup()
            	.addComponent(leftEffectPanel, GroupLayout.PREFERRED_SIZE, 100, GroupLayout.PREFERRED_SIZE)
            	.addComponent(rightEffectLayer, GroupLayout.PREFERRED_SIZE, 725, GroupLayout.PREFERRED_SIZE))           
        );
        effectPanelLayout.setVerticalGroup(
        	effectPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
            	.addComponent(leftEffectPanel, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
            	.addComponent(rightEffectLayer, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)//)
        );
        
        JPanel t = new JPanel();
        t.setBackground(Color.WHITE);
        t.setBounds(0,0,85,22);
        t.setPreferredSize(new Dimension(85,22));
        t.setBorder(new RulerBorder());
        GroupLayout tracksPanelLayout = new GroupLayout(tracksPanel);
        tracksPanel.setLayout(tracksPanelLayout);
        tracksPanelLayout.setHorizontalGroup(
            tracksPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
            	.addGroup(tracksPanelLayout.createSequentialGroup()
            			.addComponent(t, GroupLayout.DEFAULT_SIZE, 85, Short.MAX_VALUE)
            			.addComponent(rulerPane, GroupLayout.DEFAULT_SIZE, 710, Short.MAX_VALUE)
            			.addGap(19, 19, 19))
               	.addComponent(effectLayerPane, GroupLayout.DEFAULT_SIZE, 900, Short.MAX_VALUE)
               	.addGroup(tracksPanelLayout.createSequentialGroup()
               			.addGap(100, 100, 100)
               			.addComponent(trackHorizontalScroll, GroupLayout.DEFAULT_SIZE, 725, Short.MAX_VALUE))
        );
        tracksPanelLayout.setVerticalGroup(
            tracksPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
            	.addGroup(tracksPanelLayout.createSequentialGroup()
            			.addGroup(tracksPanelLayout.createParallelGroup()
            					.addComponent(t, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
            					.addComponent(rulerPane, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
            			.addComponent(effectLayerPane, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
            			.addComponent(trackHorizontalScroll, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
        );

        playback.setBorder(javax.swing.BorderFactory.createTitledBorder("Playback"));
        playback.setName("playback");
        
        playButton.setText("Play");
        playButton.setName("playButton");
        playButton.setActionCommand("playButton");
        playButton.setToolTipText("Play video");
        playButton.addActionListener(this);
        playButton.setEnabled(false);
        
        pauseButton.setText("Pause");
        pauseButton.setName("pauseButton");
        pauseButton.setActionCommand("pauseButton");
        pauseButton.addActionListener(this);
        pauseButton.setToolTipText("Pause video");
        pauseButton.setEnabled(false);

        backwardButton.setText("Backward");
        backwardButton.setName("backwardButton");
        backwardButton.setActionCommand("backwardButton");
        backwardButton.addActionListener(this);
        backwardButton.setToolTipText("Jump 5 seconds backward");
        backwardButton.setEnabled(false);

        forwardButton.setText("Forward");
        forwardButton.setName("forwardButton");
        forwardButton.setActionCommand("forwardButton");
        forwardButton.addActionListener(this);
        forwardButton.setToolTipText("Jump 5 seconds forward");
        forwardButton.setEnabled(false);

        stopButton.setText("Stop");
        stopButton.setName("stopButton");
        stopButton.setActionCommand("stopButton");
        stopButton.addActionListener(this);
        stopButton.setToolTipText("Stop video");
        stopButton.setEnabled(false);
        
        rewindButton.setText("Rewind");
        rewindButton.setName("rewindButton");
        rewindButton.setActionCommand("rewindButton");
        rewindButton.addActionListener(this);
        rewindButton.setToolTipText("Rewind video");
        rewindButton.setEnabled(false);
        
        volumeSlider.setName("volumeSlider");
        volumeSlider.setEnabled(false);
        volumeSlider.setPreferredSize(new Dimension(100, 5));
		SliderToolTips.enableSliderToolTips(volumeSlider, this);
        
        GroupLayout playbackLayout = new GroupLayout(playback);
        playback.setLayout(playbackLayout);
        playbackLayout.setHorizontalGroup(
            playbackLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGroup(playbackLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(playbackLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
                    .addGroup(playbackLayout.createSequentialGroup()
                        .addComponent(playButton)
                        .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(stopButton)
                        .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(pauseButton))
                    .addGroup(playbackLayout.createSequentialGroup()
                    	.addGap(5,5,5)
                    	.addComponent(backwardButton)
                    	.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                    	.addComponent(forwardButton))
                    .addGroup(playbackLayout.createSequentialGroup()
                    	.addGap(60,60,60)
                    	.addComponent(rewindButton))
                    .addGroup(playbackLayout.createSequentialGroup()
                    	.addComponent(volumeLabel)
                    	.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                    	.addComponent(volumeSlider)))
                .addContainerGap(GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addGap(0, 227, Short.MAX_VALUE)
        );
        
        playbackLayout.linkSize(SwingConstants.HORIZONTAL, new Component[] {backwardButton, forwardButton});
        
        playbackLayout.setVerticalGroup(
            playbackLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGroup(playbackLayout.createSequentialGroup()
                .addGroup(playbackLayout.createParallelGroup(GroupLayout.Alignment.BASELINE)
                    .addComponent(playButton)
                    .addComponent(stopButton)
                    .addComponent(pauseButton))
                .addGap(d, d, d)
                .addGroup(playbackLayout.createParallelGroup(GroupLayout.Alignment.BASELINE)
                	.addComponent(backwardButton)
                	.addComponent(forwardButton))
                .addGap(d, d, d)
                .addGroup(playbackLayout.createParallelGroup(GroupLayout.Alignment.BASELINE)
                	.addComponent(rewindButton))
                .addGap(d, d, d)
                .addGroup(playbackLayout.createParallelGroup(GroupLayout.Alignment.BASELINE)
                    .addComponent(volumeLabel)
                	.addComponent(volumeSlider)))
        );
        
        infoPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Video info"));
        infoPanel.setName("videoinfo");

        GroupLayout infoPanelLayout = new GroupLayout(infoPanel);
        infoPanel.setLayout(infoPanelLayout);
        infoPanelLayout.setHorizontalGroup(
        	infoPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGroup(infoPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(infoPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
                    .addGroup(infoPanelLayout.createSequentialGroup()
                        .addComponent(lblDuration)
                        .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(durationLabel))
                    .addGroup(infoPanelLayout.createSequentialGroup()
                        .addComponent(lblCurTime)
                        .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(curTimeLabel))
                    .addGroup(infoPanelLayout.createSequentialGroup()
                    	.addComponent(lblCodec)
                    	.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                    	.addComponent(codecLabel))
                    .addGroup(infoPanelLayout.createSequentialGroup()
                    	.addComponent(lblBitrate)
                    	.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                    	.addComponent(bitrateLabel))
                    .addComponent(lblResolution)
                    .addComponent(resolutionLabel))
                .addContainerGap(GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
            .addGap(0, 227, Short.MAX_VALUE)
        );
        infoPanelLayout.setVerticalGroup(
        	infoPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGroup(infoPanelLayout.createSequentialGroup()
                .addGroup(infoPanelLayout.createParallelGroup(GroupLayout.Alignment.BASELINE)
                    .addComponent(lblDuration)
                    .addComponent(durationLabel))
                .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(infoPanelLayout.createParallelGroup(GroupLayout.Alignment.BASELINE)
                    .addComponent(lblCurTime)
                    .addComponent(curTimeLabel))
                .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(infoPanelLayout.createParallelGroup(GroupLayout.Alignment.BASELINE)
                	.addComponent(lblCodec)
                	.addComponent(codecLabel))
                .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(infoPanelLayout.createParallelGroup(GroupLayout.Alignment.BASELINE)
                	.addComponent(lblBitrate)
                	.addComponent(bitrateLabel))
                .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(lblResolution)
                .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(resolutionLabel))
        );        

        GroupLayout mainPanelLayout = new GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
            	.addComponent(appControls, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
            	.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(videoPanel, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(mainPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
                	.addComponent(playback, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
                	.addComponent(infoPanel, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
                .addContainerGap(199, Short.MAX_VALUE))
            .addComponent(timeLineShots, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(tracksPanel, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(timeLineNormal, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addGroup(mainPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING, false)
                	.addComponent(appControls, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                	.addGroup(mainPanelLayout.createSequentialGroup()
                		.addComponent(playback, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                		.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                		.addComponent(infoPanel, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addComponent(videoPanel, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(timeLineNormal, GroupLayout.PREFERRED_SIZE, 41, GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(timeLineShots, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(tracksPanel, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGap(5, 5, 5))
        );
        
        add(mainPanel);
        
        addingEffectTracks(configElements);
        
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		setTitle(applicationName_);
		setName("SEVino");
		setResizable(false);
    }
    
    /**
	 * Method for handling the window closing operation
	 */
	public void windowClosing(WindowEvent event) {
		if(event.getID() == WindowEvent.WINDOW_CLOSING)	{
			if(JOptionPane.showConfirmDialog(null, "Are you sure?", "Exit?", JOptionPane.YES_NO_OPTION,
					JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION) {
				videoDestroy();
				
				if (shotsThread != null) {
					shotsThread.stopThread();
					shotsThread = null;
				}
				
				System.exit(0);
			}
		}
	}
		
	/**
	 * Method for handling the window iconify operation
	 */
	public void windowIconified(WindowEvent event) {}
		
	/**
	 * Method for handling the window deiconify operation
	 */
	public void windowDeiconified(WindowEvent event) {}
		
	/**
	 * Method for handling the window deactivation operation
	 */
	public void windowDeactivated(WindowEvent event){}
		
	/**
	 * Method for handling the window activation operation
	 */
	public void windowActivated(WindowEvent event) {}
		
	/**
	 * Method for handling the window open operation
	 */
	public void windowOpened(WindowEvent event) {}
	
	/**
	 * Method for handling the window closed operation
	 */
	public void windowClosed(WindowEvent event) {}
		
	/**
	 * Method for handling the actions performed
	 */
	public void actionPerformed(ActionEvent event) {
		if (event.getActionCommand().equals("openVideo"))
			performAVFile();
		else if (event.getActionCommand().equals("saveSEM"))
			performSEMFileSave();
		else if (event.getActionCommand().equals("exitApp"))
			windowClosing(new WindowEvent(this, WindowEvent.WINDOW_CLOSING));
		else if (event.getActionCommand().equals("playButton"))
			performPlayVideo();
		else if (event.getActionCommand().equals("stopButton"))
			performStopVideo();
		else if (event.getActionCommand().equals("pauseButton"))
			performPauseVideo();
		else if (event.getActionCommand().equals("forwardButton"))
			performForwardVideo();
		else if (event.getActionCommand().equals("backwardButton"))
			performBackwardVideo();
		else if (event.getActionCommand().equals("rewindButton"))
			performRewindVideo();
		else if (event.getActionCommand().equals("authorButton"))
			performAuthorInfo();
		else if (event.getActionCommand().equals("loadSEM"))
			performLoadSEM();
		else if (event.getActionCommand().equals("clearBars"))
			clearEffectBars();
		else if (event.getActionCommand().equals("uploadSEM"))
			performUploadSEM();
		else if (event.getActionCommand().equals("statisticsSEM"))
			performStatistics();
	}
	
	/**
	 * Method for destroying the video
	 */
	private void videoDestroy() {
		videoPanel.destroyVideo();
		//Runtime r = Runtime.getRuntime();
		//r.gc();
	}
	
	/**
	 * Method for handling the A/V filechooser
	 */
	private void performAVFile()
	{
		JFileChooser jfc = new JFileChooser(loadVideoPath);
		jfc.setFileFilter(new AVFileFilter());
		
		if(jfc.showOpenDialog(null) != JFileChooser.APPROVE_OPTION)
			return;
		
		if(jfc.getSelectedFile() != null) {
			loadVideoPath = jfc.getCurrentDirectory().toString();
			String videoFile = jfc.getSelectedFile().getName();
			// kill shot detection if running
			if (shotsThread != null) {
				shotsThread.stopThread();

				try {
					shotsThread.join();
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				shotsThread = null;
			}
			
			videoDestroy();
					
			forwardButton.setEnabled(false);
			backwardButton.setEnabled(false);
			pauseButton.setEnabled(false);
			rewindButton.setEnabled(false);
			stopButton.setEnabled(false);
			timeSlider.setEnabled(false);
			playButton.setEnabled(false);
			volumeSlider.setEnabled(false);
	        trackHorizontalScroll.setEnabled(false);
			curVideoFile = jfc.getSelectedFile().getPath();
			reinitBars();
			buildShotDetectionAndInitUI();
			//shotsThread = null;
			
			boolean retVal = videoPanel.init(curVideoFile);
			if (!retVal) {
				if (timer_ != null) {
					timer_.cancel();
					guiUpdater_ = null;
					timer_ = null;
				}
			}
			
			// check if sem file with same name exists and load it
			final LoadSEM loadSEM = new LoadSEM(effectTracks_);
			String semFile = jfc.getSelectedFile().getPath();
			int posExtension = semFile.lastIndexOf(".");
			if (posExtension != -1)
				semFile = semFile.substring(0, posExtension);
			
			semFile += ".xml";
			
			File testFile = new File(semFile);

			if (testFile.exists()) {
				final String file = semFile;
				
				JLabel label = new JLabel("Loading SEM...", JLabel.CENTER);
		        final JDialog dialog = new JDialog();
		        dialog.getContentPane().add(label);
		        dialog.setModal(true);
		        dialog.setSize(200,60);
		        dialog.setLocationRelativeTo(null);
		        dialog.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	        
				SwingWorker<Boolean, Object> sw = new SwingWorker<Boolean, Object>() {
					public void done() {
						dialog.dispose();
						Boolean retVal;
						try {
							retVal = get();
							if (!retVal) {
								System.out.println("Found XML-file with same name as video but wasn't able to load!");
								clearEffectBars();
							}
							else {
								effectTracks_ = loadSEM.getEffectTracks();
								authorInfo = loadSEM.getAuthorInfo();
							}
						} catch (Exception e) {
							System.out.println("Found XML-file with same name as video but wasn't able to load! - " + e.getMessage());
							clearEffectBars();
						}
					}

					protected Boolean doInBackground() throws Exception {
						return loadSEM.loadSEM(file, gui);
					}
			    };
			    
			    sw.execute();
			    dialog.setVisible(true);
			}			
			
			setTitle(applicationName_ + " - " + videoFile);
		}
	}
	
	/**
	 * Method for stopping the video
	 */
	private void performStopVideo() {
		videoPanel.stop();
		
		if (timer_ != null) {
			timer_.cancel();
			guiUpdater_ = null;
			timer_ = null;
		}
	}
	
	/**
	 * Method for starting the video
	 */
	private void performPlayVideo() {		
		if (timer_ == null && guiUpdater_ == null) {
			timer_ = new Timer();
			guiUpdater_ = new GuiUpdater(this);
			guiUpdater_.setTimeList(timeList_);
			timer_.schedule(guiUpdater_, 0, 100);
		}
		
		if (videoPanel != null)
			videoPanel.start();
		
		forwardButton.setEnabled(true);
		backwardButton.setEnabled(true);
		pauseButton.setEnabled(true);
		rewindButton.setEnabled(true);
		stopButton.setEnabled(true);
		timeSlider.setEnabled(true);
		volumeSlider.setEnabled(true);
	}
	
	/**
	 * Method for pausing the video
	 */
	private void performPauseVideo() {
		if (videoPanel != null)
			videoPanel.pause();
	}
	
	/**
	 * Method for moving 5 seconds forward the video
	 */
	private void performForwardVideo() {
		if (videoPanel != null)
			videoPanel.forward();
	}
	
	/**
	 * Method for moving 5 seconds backward the video
	 */
	private void performBackwardVideo() {
		if (videoPanel != null)
			videoPanel.backward();
	}
	
	/**
	 * Method for rewinding the video
	 */
	private void performRewindVideo() {
		if (videoPanel != null)
			videoPanel.rewind();	
	}
	
	/**
	 * Method for handling the SEM filechooser and storing SEM file
	 */
	private void performSEMFileSave() {
		JFileChooser jfc = new JFileChooser(saveSEMPath);
		jfc.setFileFilter(new SEMFileFilter());
		
		if(jfc.showSaveDialog(null) != JFileChooser.APPROVE_OPTION)
			return;
		
		if(jfc.getSelectedFile() != null){
			saveSEMPath = jfc.getCurrentDirectory().toString();
			String file = jfc.getSelectedFile().getPath();
			if (!file.toLowerCase().endsWith(".xml"))
				file += ".xml";
			
				final SaveSEM saveSEM = new SaveSEM(authorInfo, preProcessEffectTracks(effectTracks_), file);
			
				JLabel label = new JLabel("Generating SEM...", JLabel.CENTER);
		        final JDialog dialog = new JDialog();
		        dialog.getContentPane().add(label);
		        dialog.setModal(true);
		        dialog.setSize(200,60);
		        dialog.setLocationRelativeTo(null);
		        dialog.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	        
				SwingWorker<Boolean, Object> sw = new SwingWorker<Boolean, Object>() {
					public void done() {
						dialog.dispose();
						Boolean retVal;
						try {
							retVal = get();
							if (!retVal)
								JOptionPane.showMessageDialog(null, "An error occurred during SEM generation!");
							else
								JOptionPane.showMessageDialog(null, "SEM generation successful!");
						} catch (Exception e) {
							JOptionPane.showMessageDialog(null, "An error occurred during SEM generation! - " + e.getMessage());
						}
					}
	
					protected Boolean doInBackground() throws Exception {
						return saveSEM.generateSEM();
					}
			    };
			    sw.execute();
			    dialog.setVisible(true);
		}
	}
	
	/**
	 * Method for handling the SEM generation and upload to server
	 */
	private void performUploadSEM() {
		// prepare content for message box
		final JPanel content = new JPanel();
		content.setBounds(0, 0, 400, 150);
		content.setPreferredSize(new Dimension(400, 150));
		content.setLayout(new java.awt.GridLayout(0, 2));
		
		try {
			loadServerFile(SERVERINI);
		} catch (IOException e1) {
			System.out.println("Unable to load server.ini: " + e1.getMessage());
		}
		
		JLabel note = new JLabel("Upload is done via HTTP POST!");
		JLabel label = new JLabel("Servername: ");
		JLabel label2 = new JLabel("Port: ");
		JLabel label3 = new JLabel("Path: ");
		JLabel label4 = new JLabel("Filename: ");
		JLabel label5 = new JLabel("Upload Video: ");
		JTextField uploadServer = new JTextField();
		uploadServer.setPreferredSize(new Dimension(150, 20));
		uploadServer.setText(preSetServerAddress);
		IntField uploadServerPort = new IntField(preSetServerPort, 80);
		JTextField uploadPath = new JTextField();
		uploadPath.setPreferredSize(new Dimension(150, 20));
		uploadPath.setText(preSetServerPath);
		JTextField uploadFilename = new JTextField();
		uploadFilename.setPreferredSize(new Dimension(150, 20));
		uploadFilename.setText(preSetFilename);
		JCheckBox uploadVideo = new JCheckBox();
		uploadVideo.setPreferredSize(new Dimension(50,20));
		
		content.add(label);
		content.add(uploadServer);
		content.add(label2);
		content.add(uploadServerPort);
		content.add(label3);
		content.add(uploadPath);
		content.add(label4);
		content.add(uploadFilename);
		content.add(label5);
		content.add(uploadVideo);
		content.add(new JLabel(""));
		content.add(new JLabel(""));
		content.add(note);
		content.add(new JLabel(""));
		
		int retVal = JOptionPane.showConfirmDialog(this, content, "Enter server settings", JOptionPane.OK_CANCEL_OPTION);
		if (retVal == JOptionPane.OK_OPTION) {
			if (!uploadServer.getText().equals("") && !uploadServerPort.getText().equals("") && !uploadPath.getText().equals("") &&
					!uploadFilename.getText().equals("")) {
				if (!uploadFilename.getText().toLowerCase().endsWith(".xml"))
					preSetFilename = uploadFilename.getText() + ".xml";
				else
					preSetFilename = uploadFilename.getText();
				
				boolean success = true;
				
				preSetServerAddress = uploadServer.getText();
				preSetServerPath = uploadPath.getText();
				preSetServerPort = uploadServerPort.getValue();
								
				final SaveSEM saveSEM = new SaveSEM(authorInfo, preProcessEffectTracks(effectTracks_), preSetFilename);
				
				JLabel labelSEM = new JLabel("Generating SEM...", JLabel.CENTER);
		        final JDialog dialog = new JDialog();
		        dialog.getContentPane().add(labelSEM);
		        dialog.setModal(true);
		        dialog.setSize(200,60);
		        dialog.setLocationRelativeTo(null);
		        dialog.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	        
				SwingWorker<Boolean, Object> sw = new SwingWorker<Boolean, Object>() {
					public void done() {
						dialog.dispose();
						Boolean retVal;
						try {
							retVal = get();
							if (!retVal)
								JOptionPane.showMessageDialog(null, "An error occurred during SEM generation!");
						} catch (Exception e) {
							JOptionPane.showMessageDialog(null, "An error occurred during SEM generation! - " + e.getMessage());
						}
					}
	
					protected Boolean doInBackground() throws Exception {
						return saveSEM.generateSEM(preSetServerAddress, preSetServerPort, preSetServerPath, true);
					}
			    };
			    sw.execute();
			    dialog.setVisible(true);
			    
			    try {
					success = sw.get();
				} catch (Exception e1) {
					e1.printStackTrace();
				}
			    
			    // save optional video if necessary
				if (success && uploadVideo.isSelected()) {
					try {
						String videoFile = "video.avi";
						int pos = curVideoFile.lastIndexOf(System.getProperty("file.separator"));
						if (pos != -1)
							videoFile = curVideoFile.substring(pos+1);

						UploadFile.upload(preSetServerAddress, preSetServerPort, preSetServerPath, videoFile, curVideoFile);
					} catch (IOException e) {
						JOptionPane.showMessageDialog(null, "An error occured during upload!");
						success = false;
					}
				}
				
				try {
					storeServerFile(SERVERINI);
				} catch (IOException e) {
					System.out.println("Unable to store server.ini: " + e.getMessage());
				}
				
				if (success)
					JOptionPane.showMessageDialog(null, "Upload successful!");
				else
					JOptionPane.showMessageDialog(null, "An error occured during upload!");
			}
			else
				JOptionPane.showMessageDialog(null, "Server address cannot be empty!");
		}
	}
		
	/**
	 * Method for building the shot detection and updating the UI
	 */
	private void buildShotDetectionAndInitUI() {
		progressBar.setValue(0);
		timeSlider.setMaximum(0);
		timeSlider.setValue(0);
		volumeSlider.setValue(100);
		curTimeLabel.setText("00:00:00");
		durationLabel.setText("00:00:00");
		bitrateLabel.setText("");
		codecLabel.setText("");
		resolutionLabel.setText("");			
		//videoPanel.setVideoDuration(0);
		shotPanels.removeAllElements();
		shotPanel.removeAll();
		shotPanel.setPreferredSize(new Dimension(800, 50));
		shotPanel.repaint();
		trackHorizontalScroll.setValue(0);	
		
		//System.out.println(shotsThread);
		shotsThread = new ShotsThread(curVideoFile, gui);
		shotsThread.setPriority(10); // give it a high priority
		shotsThread.start();
	}
	
	/**
	 * Method for updating the progress bar
	 * @param percent to set the progress bar
	 */
	public synchronized void updateProgressBar(int percent) {
       	 progressBar.setValue(percent);
	}

	/**
	 * Method for handling mouse click
	 * @param e MouseEvent to handle
	 */
	public void mouseClicked(MouseEvent e) { }

	/**
	 * Method for handling mouse entered
	 * @param e MouseEvent to handle
	 */
	public void mouseEntered(MouseEvent e) { }

	/**
	 * Method for handling mouse exited
	 * @param e MouseEvent to handle
	 */
	public void mouseExited(MouseEvent e) { }

	/**
	 * Method for handling mouse pressed
	 * @param e MouseEvent to handle
	 */
	public void mousePressed(MouseEvent e) { }

	/**
	 * Method for handling mouse release
	 * @param e MouseEvent to handle
	 */
	public void mouseReleased(MouseEvent e) {
		if (e.getSource() instanceof Painter) {
			if (e.getButton() == MouseEvent.BUTTON1) {
				String controlName = ((Painter)e.getSource()).getName();
				if (controlName.indexOf("shot") > -1) {
					double time = Double.valueOf(controlName.substring(4));
					videoPanel.setPlayerTime(time);
				}
			}
		}
	}
	
	/**
	 * Method for setting the shot border
	 * @param time double where to set it
	 */
	public void setShotBorder(double time) {
		for (int i = 0; i < shotPanels.size(); i++) {
			if (shotPanels.elementAt(i).getName().equals("shot" + time))
				shotPanels.elementAt(i).setBorder(new EtchedBorder(Color.RED, Color.RED));
			else
				shotPanels.elementAt(i).setBorder(null);
		}
	}

	/**
	 * Method for handling the scroll bar adjustment
	 * @param e AdjustmentEvent to handle
	 */
	public void adjustmentValueChanged(AdjustmentEvent e) {
		JScrollBar jb = (JScrollBar)e.getSource();
	
		if (jb == trackHorizontalScroll) {
			int value = e.getValue();
			rightEffectLayer.getHorizontalScrollBar().setValue(value);
			rulerPane.getHorizontalScrollBar().setValue(value);
		}
	}
	
	/**
	 * Method for showing and handling the AuhtorInformationWindow
	 */
	private void performAuthorInfo() {
		AuthorInformationWindow aiw = null;
		
		if (authorInfo == null)
			aiw = new AuthorInformationWindow();
		else
			aiw = new AuthorInformationWindow(authorInfo.getCreatorName(), authorInfo.getCreatorOrganization(), 
					authorInfo.getVersion(), authorInfo.getConfidence(), authorInfo.getComment());
		
		if (aiw.showDialog() == 1) {
			authorInfo = new AuthorInfo();
			
			if (aiw.getComment() != null && !aiw.getComment().equals("Enter here your comments"))
				authorInfo.setComment(aiw.getComment());
			
			authorInfo.setConfidence(aiw.getConfidence());
			authorInfo.setCreatorName(aiw.getCreatorName());
			authorInfo.setCreatorOrganization(aiw.getCreatorOrganization());
			authorInfo.setVersion(aiw.getVersion());
		}
	}
	
	/**
	 * Method for adding effect tracks to the GUI
	 * @param configElements Vector with config elements
	 */
	private void addingEffectTracks(Vector<ConfigElement> configElements) {
		effectTracks_ = new Vector<EffectTrack>();
		int effects = 0;
		
		for (int i = 0; i < configElements.size(); i++) {
			ConfigElement ce = configElements.elementAt(i);
			
			JPanel name = new JPanel();
			name.setName("name" + ce.getEffectName());
			name.setPreferredSize(new Dimension(100, 20));
			name.setBounds(0, 0, 100, 20);
			name.add(new JLabel(ce.getDisplayName()));
			name.setBorder(javax.swing.BorderFactory.createEtchedBorder());
			FlowLayout fl = new FlowLayout(FlowLayout.CENTER, 0, 0);
			name.setLayout(fl);

			Color eColor = ce.getEffectColor();
			Color sColor = ce.getSelectColor();

			if (eColor == null)
				eColor = Color.blue;

			if (sColor == null)
				sColor = Color.red;

			final EffectPanel bar = new EffectPanel(eColor, sColor, ce);
			bar.setPreferredSize(new Dimension(721, 20));
			bar.setBounds(0, 0, 721, 20);
			bar.setName("bar" + ce.getEffectName());
			bar.setNameOfEffect(ce.getEffectName());
			bar.setBorder(javax.swing.BorderFactory.createEtchedBorder());
			bar.addMouseListener(new MouseAdapter() {
				public void mousePressed(MouseEvent arg0) {
					if (arg0.getButton() == MouseEvent.BUTTON1)
						bar.initColorRect(arg0.getX());
				}

				public void mouseReleased(MouseEvent arg0) {
					bar.addColorRect();
				}
			});
			bar.addMouseMotionListener(new MouseMotionAdapter() {
				public void mouseDragged(MouseEvent arg0) {
					bar.drawColorRect(arg0.getX());

					if (arg0.getX() > (bar.getVisibleRect().x + bar
							.getVisibleRect().width))
						trackHorizontalScroll.setValue(trackHorizontalScroll
								.getValue() + 7);
					else if (arg0.getX() < bar.getVisibleRect().x)
						trackHorizontalScroll.setValue(trackHorizontalScroll
								.getValue() - 7);
				}
			});
			bar.setLayout(fl);

			leftEffectPanel.add(name);
			rightEffectPanel.add(bar);
			effectTracks_.add(new EffectTrack(name, bar));
			effects++;
		}
		
		int height = effects*20;		
		
		if (height > 240) {
			effectPanel.setPreferredSize(new Dimension(835, height));
			leftEffectPanel.setPreferredSize(new Dimension(100, height));
			rightEffectPanel.setPreferredSize(new Dimension(721, height));
			rightEffectLayer.setPreferredSize(new Dimension(730, height+3));
		}
	}
	
	/**
	 * Method for loading a SEM file and handling the file chooser
	 */
	private void performLoadSEM() {
		JFileChooser jfc = new JFileChooser(loadSEMPath);
		jfc.setFileFilter(new SEMFileFilter());
		
		if(jfc.showOpenDialog(null) != JFileChooser.APPROVE_OPTION)
			return;
		
		clearEffectBars();
		if(jfc.getSelectedFile() != null) {
			loadSEMPath = jfc.getCurrentDirectory().toString();
			final LoadSEM loadSEM = new LoadSEM(effectTracks_);
			final String file = jfc.getSelectedFile().getPath();
			
			JLabel label = new JLabel("Loading SEM...", JLabel.CENTER);
	        final JDialog dialog = new JDialog();
	        dialog.getContentPane().add(label);
	        dialog.setModal(true);
	        dialog.setSize(200,60);
	        dialog.setLocationRelativeTo(null);
	        dialog.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        
			SwingWorker<Boolean, Object> sw = new SwingWorker<Boolean, Object>() {
				public void done() {
					dialog.dispose();
					Boolean retVal;
					try {
						retVal = get();
						if (!retVal) {
							JOptionPane.showMessageDialog(null, "An error occurred during SEM loading!");
							clearEffectBars();
						}
						else {
							effectTracks_ = loadSEM.getEffectTracks();
							authorInfo = loadSEM.getAuthorInfo();
						}
					} catch (Exception e) {
						JOptionPane.showMessageDialog(null, "An error occurred during SEM loading! - " + e.getMessage());
					}
				}

				protected Boolean doInBackground() throws Exception {
					return loadSEM.loadSEM(file, gui);
				}
		    };
		    sw.execute();
		    dialog.setVisible(true);
		}
	}
	
	/**
	 * Method for reinitializing the effect panels
	 */
	private void reinitBars() {
		for (int i = 0; i < rightEffectPanel.getComponentCount(); i++) {
			Component c = rightEffectPanel.getComponent(i);
			if (c instanceof EffectPanel)
				((EffectPanel)c).clearBar();
		}
		
		rightEffectPanel.setBounds(0, 0, 720, rightEffectPanel.getHeight());
		rightEffectPanel.setPreferredSize(new Dimension(720, rightEffectPanel.getHeight()));
		rightEffectPanel.revalidate();
		
		ruler.setPassedTime(0);
		ruler.repaint();
	}
	
	/**
	 * Method for clearing the effect panels
	 */
	private void clearEffectBars() {
		for (int i = 0; i < rightEffectPanel.getComponentCount(); i++) {
			Component c = rightEffectPanel.getComponent(i);
			if (c instanceof EffectPanel)
				((EffectPanel)c).clearBar();
		}

		rightEffectPanel.revalidate();
	}
	
	/**
	 * Method for pre-processing the effect tracks by putting all effect bars of the same track into one vector
	 * @param effectTracks Vector with the effect tracks
	 * @return Hashtable<String, Vector<BarElement>> with the pre-processed effect tracks
	 */
	private Hashtable<String, Vector<BarElement>> preProcessEffectTracks(Vector<EffectTrack> effectTracks) {
		Hashtable<String, Vector<BarElement>> semEffectTable = new Hashtable<String, Vector<BarElement>>();
		for (int i = 0; i < effectTracks.size(); i++) {
			EffectTrack et = effectTracks.elementAt(i);
			EffectPanel be = et.getEffectBarPanel();
			semEffectTable.put(be.getNameOfEffect(), be.getBars());
		}
		
		return semEffectTable;
	}
	
	/**
	 * Method for storing entered upload settings
	 * @param file String with filename to store to
	 * @throws IOException thrown if an io error occurred
	 */
	private void storeServerFile(String file) throws IOException {
		File f = new File(file);
		if (f.exists())
			f.delete();
		
		FileWriter wr = new FileWriter(f);
		wr.write("Address:" + preSetServerAddress + "\n");
		wr.write("Port:" + preSetServerPort + "\n");
		wr.write("Path:" + preSetServerPath + "\n");
		wr.write("Filename:" + preSetFilename + "\n");
		wr.flush();
		wr.close();
	}
	
	/**
	 * Method for loading entered upload settings
	 * @param file String with filename to load from
	 * @throws IOException thrown if an io error occurred
	 */
	private void loadServerFile(String file) throws IOException {
		File f = new File(file);
		if (f.exists()) {
			FileReader fr = new FileReader(f);
			
			int c = -1;
			String key = "";
			String value = "";
			boolean keyFound = false;
			while ((c = fr.read()) != -1) {
				if (((char)c) != ':' && !keyFound)
					key += (char)c;
				else if (((char)c) == ':')
					keyFound = true;
				else if (((char)c) != '\n')
					value += (char)c;
				else {
					if (key.equals("Address"))
						preSetServerAddress = value;
					else if (key.equals("Port"))
						preSetServerPort = Integer.parseInt(value);
					else if (key.equals("Path"))
						preSetServerPath = value;
					else if (key.equals("Filename"))
						preSetFilename = value;
					
					keyFound = false;
					key = "";
					value = "";
				}
			}
			fr.close();
		}
		else {
			preSetServerAddress = "localhost";
		    preSetServerPort = 8080;
		    preSetServerPath = "/cms/fileUpload.do";
		    preSetFilename = "testsem.xml";
		}
	}
	
	/**
	 * This method counts the number of effects for each effect type
	 */
	private void performStatistics() {
		String msgBoxText = "Annotated Effects:\n\n";
		boolean effectAnnot = false;
		for (int i = 0; i < rightEffectPanel.getComponentCount(); i++) {
			Component c = rightEffectPanel.getComponent(i);
			if (c instanceof EffectPanel) {
				EffectPanel ep = (EffectPanel)c;
				if (ep.getBarCount() > 0) {
					msgBoxText += ep.getNameOfEffect();
					msgBoxText += ": ";
					msgBoxText += ep.getBarCount();
					msgBoxText += "\n";
					
					effectAnnot = true;
				}
			}
		}
	
		if (effectAnnot)
			JOptionPane.showMessageDialog(null, msgBoxText, "SEVino - Statistics", JOptionPane.INFORMATION_MESSAGE);
		else
			JOptionPane.showMessageDialog(null, "There are no effects annotated!", "SEVino - Statistics", JOptionPane.INFORMATION_MESSAGE);
	}
}
