/*
 * This file is part of SEVino.
 *
 * SEVino is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEVino is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SEVino.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

package io;

import java.io.IOException;
import java.util.Vector;
import javax.xml.bind.JAXBException;
import lib.ConfigElement;
import xml.ConfigHandler;

/**
 * Class for loading the config file for SEVino
 * 
 * @author Markus Waltl
 * @version 1.0
 */
public class LoadConfig {
	private boolean alreadyTried = false;
	private ConfigHandler ch_ = null;
	
	/**
	 * Constructor
	 */
	public LoadConfig() {
		ch_ = new ConfigHandler();
	}
	
	/**
	 * Method for loading the config file
	 * @param configFile String with the path and filename
	 * @return Vector<ConfigElement> with found config elements
	 */
	public Vector<ConfigElement> load(String configFile) {
		if (configFile.equals(""))
			configFile = reinitConfig();
		
		try {    		
			return ch_.parse(configFile); 
		} catch (IOException e) {
			System.out.println(e.getMessage());
			
			// retry with original config
			if (!alreadyTried) {
				alreadyTried = true;
				return load(reinitConfig());
			}
			else
				return null;
		} catch (JAXBException e) {
			System.out.println(e.getMessage());
			
			// retry with original config
			if (!alreadyTried) {
				alreadyTried = true;
				return load(reinitConfig());
			}
			else
				return null;
		}
	}
	
	/**
	 * Method for reinitializing the config loader
	 * @return String with default config path
	 */
	private String reinitConfig() {
		String configFile = "";
		
		String OS = System.getProperty("os.name").toLowerCase();
		if (OS.indexOf("windows") > -1)
			configFile = "configs\\config.xml";
		else
			configFile = "configs/config.xml";
		
		return configFile;
	}
}
