/*
 * This file is part of SEVino.
 *
 * SEVino is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEVino is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SEVino.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package lib;

import java.util.Vector;

/**
 * Class which represents a container for attributes
 * 
 * @author Markus Waltl
 * @version 1.0
 */
public class Attribute {
	private Vector<String[]> attributes_ = null;
	private Vector<Vector<String[]>> enumerations_ = null;
	
	/**
	 * Constructor
	 */
	public Attribute() {
		attributes_ = new Vector<String[]>();
	}
	
	/**
	 * Constructor
	 * @param attributes Vector with attributes
	 * @param enumeration Vector with enumerations
	 */
	public Attribute(Vector<String[]> attributes, Vector<String[]> enumeration) {
		this();
		
		if (attributes == null || attributes.isEmpty())
			return;
		
		attributes_ = attributes;
		
		if (enumeration != null && !enumeration.isEmpty())
			addEnumeration(enumeration);			
	}
	
	/**
	 * Method for retrieving the attributes
	 * @return Vector<String[]> with attributes
	 */
	public Vector<String[]> getAttributes() {
		return attributes_;
	}
	
	/**
	 * Method for adding an attribute
	 * @param attribute String[] with attribute (0 = attribute name, 1 = attribute value)
	 * @return true if added successfully else false
	 */
	public boolean addAttribute(String[] attribute) {
		if (attribute == null)
			return false;
		
		attributes_.add(attribute);
		return true;
	}
	
	/**
	 * Method for adding an enumeration
	 * @param enumeration Vector with enumeration
	 * @return true if added successfully else false
	 */
	public boolean addEnumeration(Vector<String[]> enumeration) {
		if (enumeration == null || enumeration.isEmpty())
			return false;
		
		if (enumerations_ == null)
			enumerations_ = new Vector<Vector<String[]>>();
		
		enumerations_.add(enumeration);
		return true;
	}
	
	/**
	 * Method for retrieving enumerations
	 * @return Vector<Vector<String[]>> with enumerations
	 */
	public Vector<Vector<String[]>> getEnumeration() {
		return enumerations_;
	}
	
	/**
	 * Method for getting a String representation of the class
	 * @return String with representation
	 */
	public String toString() {
		String s = "";
		for (int i = 0; i < attributes_.size(); i++) {
			String[] att = attributes_.elementAt(i);
			s += att[0] + ": " + att[1] + "; ";
		}
		
		s += "\n";
		
		if (enumerations_ != null) {
			s += "Enumeration:\n";
			for (int i = 0; i < enumerations_.size(); i++) {
				Vector<String[]> v = enumerations_.elementAt(i);
				
				for (int j = 0; j < v.size(); j++) {
					String[] att = v.elementAt(j);
					s += att[0] + ": " + att[1] + "; ";
				}
				
				s += "\n";
			}

			s += "\n";
		}		
		
		return s;
	}
}
