/*
 * This file is part of SEVino.
 *
 * SEVino is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEVino is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SEVino.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package lib;

/**
 * Class for storing author information
 * 
 * @author Markus waltl
 * @version 1.0
 */
public class AuthorInfo {
	private float confidence_ = 0.0f;
	private String version_ = "";
	private String lastUpdate_ = ""; // fill during SEM creation
	private String comment_ = "";
	private String creatorName_ = "";
	private String creatorOrganization_ = "";
	private String instrument_ = "SEVino 1.0";
	private String creationTime_ = ""; // fill during SEM creation or leave if already filled
	
	/**
	 * Constructor
	 */
	public AuthorInfo() { }

	/**
	 * Method for retrieving the confidence
	 * @return float with confidence
	 */
	public float getConfidence() {
		return confidence_;
	}

	/**
	 * Method for setting the confidence
	 * @param confidence float with confidence
	 */
	public void setConfidence(float confidence) {
		this.confidence_ = confidence;
	}

	/**
	 * Method for retrieving the version
	 * @return String with the version
	 */
	public String getVersion() {
		return version_;
	}

	/**
	 * Method for setting the version
	 * @param version String with the version
	 */
	public void setVersion(String version) {
		this.version_ = version;
	}

	/**
	 * Method for retrieving the last update time
	 * @return String with the time
	 */
	public String getLastUpdate() {
		return lastUpdate_;
	}

	/**
	 * Method for setting the last update time
	 * @param lastUpdate String with the time
	 */
	public void setLastUpdate(String lastUpdate) {
		this.lastUpdate_ = lastUpdate;
	}

	/**
	 * Method for retrieving the comment
	 * @return String with the comment
	 */
	public String getComment() {
		return comment_;
	}

	/**
	 * Method for setting the comment
	 * @param comment String with the comment
	 */
	public void setComment(String comment) {
		this.comment_ = comment;
	}

	/**
	 * Method for retrieving the creator name
	 * @return String with the name
	 */
	public String getCreatorName() {
		return creatorName_;
	}

	/**
	 * Method for setting the creator name
	 * @param creatorName String with the name
	 */
	public void setCreatorName(String creatorName) {
		this.creatorName_ = creatorName;
	}

	/**
	 * Method for retrieving the creator organization
	 * @return String with the organization
	 */
	public String getCreatorOrganization() {
		return creatorOrganization_;
	}

	/**
	 * Method for setting the creator organization
	 * @param creatorOrganization String the the organization
	 */
	public void setCreatorOrganization(String creatorOrganization) {
		this.creatorOrganization_ = creatorOrganization;
	}

	/**
	 * Method for retrieving the used instrument (tool)
	 * @return String with instrument
	 */
	public String getInstrument() {
		return instrument_;
	}

	/**
	 * Method for setting the instrument (tool)
	 * @param instrument String with instrument
	 */
	public void setInstrument(String instrument) {
		this.instrument_ = instrument;
	}

	/**
	 * Method for retrieving the creation time
	 * @return String with time
	 */
	public String getCreationTime() {
		return creationTime_;
	}

	/**
	 * Method for setting the creation time
	 * @param creationTime String with time
	 */
	public void setCreationTime(String creationTime) {
		this.creationTime_ = creationTime;
	}
	
	/**
	 * Method for getting a String with the representation of the class
	 * @return String with representation
	 */
	public String toString() {
		return getStringRepresentation().toString();
	}
	
	/**
	 * Method for getting a String representation of the content
	 * @return StringBuffer with representation
	 */
	public StringBuffer getStringRepresentation() {
		StringBuffer sb = new StringBuffer("");
		
		sb.append("<DescriptionMetadata xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns='urn:mpeg:mpeg-v:2010:01-SEDL-NS' xmlns:mpeg7='urn:mpeg:mpeg7:schema:2004'>");
	 	sb.append("<mpeg7:Confidence>");
	 	sb.append(confidence_);
	 	sb.append("</mpeg7:Confidence>");
	 	
	 	if (version_ != null && !version_.equals("")) {
		 	sb.append("<mpeg7:Version>");
		 	sb.append(version_);
		 	sb.append("</mpeg7:Version>");
		}
	 	
	 	sb.append("<mpeg7:LastUpdate>");
	 	sb.append(lastUpdate_);
	 	sb.append("</mpeg7:LastUpdate>");
	 	
	 	if (comment_ != null && !comment_.equals("")) {
		 	sb.append("<mpeg7:Comment>");
		 	sb.append("<mpeg7:FreeTextAnnotation>");
		 	sb.append(comment_);
		 	sb.append("</mpeg7:FreeTextAnnotation>");
		 	sb.append("</mpeg7:Comment>");
	 	}
	 	
	 	sb.append("<mpeg7:Creator>");
	 	
	 	if (creatorName_ != null && !creatorName_.equals("")) {
		 	sb.append("<mpeg7:Role href='urn:mpeg:mpeg7:cs:MPEG7RoleCS:PERFORMER'>");
		 	sb.append("<mpeg7:Name>");
		 	sb.append(creatorName_);
		 	sb.append("</mpeg7:Name>");
		 	sb.append("</mpeg7:Role>");
	 	}
	 	
	 	if (creatorOrganization_ != null && !creatorOrganization_.equals("")) {
		 	sb.append("<mpeg7:Agent xsi:type='mpeg7:OrganizationType'>");
		 	sb.append("<mpeg7:Name>");
		 	sb.append(creatorOrganization_);
		 	sb.append("</mpeg7:Name>");
		 	sb.append("</mpeg7:Agent>");
	 	}
	 	
	 	sb.append("<mpeg7:Instrument>");
	 	sb.append("<mpeg7:Tool>");
	 	sb.append("<mpeg7:Name>");
	 	sb.append(instrument_);
	 	sb.append("</mpeg7:Name>");
	 	sb.append("</mpeg7:Tool>");
	 	sb.append("</mpeg7:Instrument>");
	 	sb.append("</mpeg7:Creator>");
	 	sb.append("<mpeg7:CreationTime>");
	 	sb.append(creationTime_);
	 	sb.append("</mpeg7:CreationTime>");
	 	sb.append("</DescriptionMetadata>");
		
		return sb;
	}	
}
