/*
 * This file is part of SEVino.
 *
 * SEVino is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEVino is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SEVino.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package lib;

import java.awt.Color;
import java.awt.Rectangle;

/**
 * Class which represents a color rectangle
 * 
 * @author Markus Waltl
 * @version 1.0
 */
public class ColorRectangle extends Rectangle implements Comparable<ColorRectangle> {
	private static final long serialVersionUID = 7906982317853583592L;
	private Color color = null;
	private String identifier = "";
	
	/**
	 * Constructor
	 * @param identifier String with rectangle identifier
	 */
	public ColorRectangle(String identifier) {
		super();
		this.identifier = identifier;
	}
	
	/**
	 * Constructor
	 * @param x int position in x-axes
	 * @param y int position in y-axes
	 * @param width int width of rectangle
	 * @param height int height of rectangle
	 * @param identifier String with rectangle identifier
	 */
	public ColorRectangle(int x, int y, int width, int height, String identifier) {
		super(x, y, width, height);
		this.identifier = identifier;
	}
	
	/**
	 * Constructor
	 * @param x int position in x-axes
	 * @param y int position in y-axes
	 * @param width int width of rectangle
	 * @param height int height of rectangle
	 * @param c Color to use for rectangle
	 * @param identifier String with rectangle identifier
	 */
	public ColorRectangle(int x, int y, int width, int height, Color c, String identifier) {
		super(x, y, width, height);
		color = c;
		this.identifier = identifier;
	}
	
	/**
	 * Method for retrieving the identifier
	 * @return String with identifier
	 */
	public String getIdentifier() {
		return identifier;
	}
	
	/**
	 * Method for setting the color of the rectangle
	 * @param c Color to use
	 */
	public void setColor(Color c) {
		color = c;
	}
	
	/**
	 * Method for retrieving the color of the rectangle
	 * @return Color of the rectangle
	 */
	public Color getColor() {
		return color;
	}

	/**
	 * Method for comparing two color rectangles
	 * @param cr ColorRectangle to compare with
	 * @return 0 if equal, -1 or 1 if not equal
	 */
	public int compareTo(ColorRectangle cr) {
		int retVal = 0;			
		if (this.x > cr.x)
			retVal = 1;
		else if (this.x < cr.x)
			retVal = -1;
				
		return retVal;
	}
}
