/*
 * This file is part of SEVino.
 *
 * SEVino is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEVino is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SEVino.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package lib;

import java.text.DecimalFormat;
import java.util.TimerTask;
import java.util.Vector;
import javax.swing.event.ChangeListener;
import app.SEVinoView;

/**
 * GuiUpdater 
 * 
 * defines an own timer for settings the time in the GUI
 * 
 * @author Markus Waltl (Alpen-Adria-Universitaet Klagenfurt)
 * @version 1.0
 */
public class GuiUpdater extends TimerTask {
	private SEVinoView gui_ = null;
	private double timeCounter = 0.0;
	private boolean isUserChanged = false;
	private Vector<Double> timeList = null;
	
	/**
	 * Constructor
	 * @param gui SEVinoView for output
	 */
	public GuiUpdater(SEVinoView gui) {
		gui_ = gui;
	}

	/**
	 * Thread running method
	 */
	public void run() {
		timeCounter = gui_.videoPanel.getCurrentVideoTime();
		
		if (timeCounter > -1) {
			int hours = 0;
			int minutes = 0;
			double seconds = timeCounter;
			minutes = (int) seconds / 60;
			seconds = seconds % 60;
			hours = minutes / 60;
			minutes = minutes % 60;
			
			DecimalFormat df2 = new DecimalFormat("#########0.0");
			String value = df2.format(seconds);
			value = value.replace(",", ".");
			seconds = new Double(value).doubleValue();
			
			value = df2.format(timeCounter);
			value = value.replace(",", ".");
			timeCounter = new Double(value).doubleValue();
	
			String length = "";
			if (hours < 10)
				length = "0" + hours + ":";
			else
				length = hours + ":";
	
			if (minutes < 10)
				length += "0" + minutes + ":";
			else
				length += minutes + ":";
	
			if (seconds < 10)
				length += "0" + seconds;
			else
				length += seconds;
	
			gui_.curTimeLabel.setText(length);
	
			if (!isUserChanged) {
				ChangeListener[] listeners = gui_.timeSlider.getChangeListeners();
				gui_.timeSlider.removeChangeListener(listeners[0]);
				gui_.timeSlider.setValue((int) (timeCounter * 10));
				gui_.timeSlider.addChangeListener(listeners[0]);
			}
	
			if (timeList != null) {
				// it can happen that a border is not set due to a over jumped
				// time stamp
				if (timeList.contains(new Double(timeCounter)))
					gui_.setShotBorder(timeCounter);
			}
			
			gui_.ruler.setPassedTime((int)(timeCounter*70));
			gui_.ruler.repaint();
		}
	}
	
	/**
	 * Method for indicating that the user is currently changing values
	 * @param changing boolean true if user is changing else false
	 */
	public void setUserChange(boolean changing) {
		isUserChanged = changing;
	}
	
	/**
	 * Method for setting the time list of the shots
	 * @param times Vector<Double> with shot times
	 */
	public void setTimeList(Vector<Double> times) {
		timeList = times;
	}
}
